package bsp

import (
	"github.com/jezek/xgb/xproto"
	"github.com/jezek/xgbutil/ewmh"
	"github.com/jezek/xgbutil/icccm"
	"github.com/jezek/xgbutil/xwindow"
	"tuxpa.in/a/zlog/log"
)

type Client struct {
	win   *xwindow.Window
	winId xproto.Window

	name string
}

// registers a client to be obtained later
func (xwm *XWM) RegisterClient(wid xproto.Window) *Client {
	c := &Client{
		winId: wid,
	}
	xwm.X.Grab()
	defer xwm.X.Ungrab()
	// dont duplicate registration
	if xwm.FindClient(wid) != nil {
		log.Trace().Any("id", wid).Msg("duplicate client registration")
		return nil
	}
	// ok this is a new one, so we need to make a new  client
	c.win = xwindow.New(xwm.X, wid)
	if _, err := c.win.Geometry(); err != nil {
		log.Err(err).Any("id", wid).Msg("get geometry")
		return nil
	}
	err := xproto.ChangeSaveSetChecked(xwm.X.Conn(), xproto.SetModeInsert, c.winId).Check()
	if err != nil {
		log.Err(err).Any("id", wid).Msg("change save set checked")
		return nil
	}
	xwm.initClient(c)
	return c
}

func (xwm *XWM) initClient(c *Client) {
	xwm.updateName(c)
}
func (xwm *XWM) updateName(c *Client) {
	newName := func() (n string) {
		n, _ = ewmh.WmNameGet(xwm.X, c.winId)
		if len(n) > 0 {
			return
		}

		n, _ = icccm.WmNameGet(xwm.X, c.winId)
		if len(n) > 0 {
			return
		}
		return
	}()

	if newName != c.name {
		c.name = newName
		ewmh.WmVisibleNameSet(xwm.X, c.winId, c.name)
	}
}

func (xwm *XWM) FindClient(wid xproto.Window) (c *Client) {
	xwm.W.View(func() error {
		val, ok := xwm.W.Clients[wid]
		if ok {
			c = val
		}
		return nil
	})
	return
}
