package bsp

import (
	"bytes"
	"encoding/binary"
	"encoding/hex"
	"fmt"
	"sync"

	"github.com/jezek/xgb/randr"
	"github.com/jezek/xgb/xproto"
	"github.com/jezek/xgbutil/xwindow"
	"tuxpa.in/t/wm/src/bsp/cfg"
)

type WM struct {
	Desktops []*Desktop
	Monitors []*Monitor

	Cfg *cfg.Modifier[*Config]

	Root *xwindow.Window

	Clients map[xproto.Window]*Client

	mu sync.RWMutex
}

type Desktop struct {
	Name    string
	Monitor *Monitor

	Cfg *cfg.Modifier[*Config]
}

type Monitor struct {
	Name string
	Id   xproto.Monitor

	raw *randr.GetOutputInfoReply

	Cfg *cfg.Modifier[*Config]
}

func (m *Monitor) HexId() []byte {
	o := bytes.NewBuffer([]byte("0x"))
	binary.Write(hex.NewEncoder(o), binary.LittleEndian, m.Id)
	return o.Bytes()
}

type Node struct {
	Name string

	Cfg *cfg.Modifier[*Config]
}

func (w *WM) Mutate(fn func() error) error {
	w.mu.Lock()
	defer w.mu.Unlock()
	if fn != nil {
		return fn()
	}
	return nil
}

func (w *WM) View(fn func() error) error {
	w.mu.RLock()
	defer w.mu.RUnlock()
	if fn != nil {
		return fn()
	}
	return nil
}

func (w *WM) AddDesktop(name string, monitorName string) error {
	return w.Mutate(func() error {
		var monitor *Monitor
		for _, v := range w.Monitors {
			if v.Name == monitorName {
				monitor = v
			}
		}
		if monitor == nil {
			return fmt.Errorf("invalid descriptor found in '%s'", monitorName)
		}
		w.Desktops = append(w.Desktops, &Desktop{
			Name:    name,
			Cfg:     cfg.NewModifier(&Config{}),
			Monitor: monitor,
		})
		return nil
	})
}

func NewWM() *WM {
	w := &WM{
		Cfg:     cfg.NewModifier(&Config{}),
		Clients: make(map[xproto.Window]*Client),
	}
	w.Cfg.FillDefaults()

	return w
}
