package copies

import "fmt"

type ErrInvalidArgumentCount struct {
	Name  string
	Value int
}

func (u *ErrInvalidArgumentCount) Error() string {
	return fmt.Sprintf(`Was expecting %s arguments, received %d.\n`, u.Name, u.Value)
}

type ErrInvalidValue struct {
	Name  string
	Value string
}

func (u *ErrInvalidValue) Error() string {
	return fmt.Sprintf(`%s: Invalid value: '%s'.\n`, u.Name, u.Value)
}

func NewInvalidValueErr(n, v string) *ErrInvalidValue {
	return &ErrInvalidValue{
		Name:  n,
		Value: v,
	}
}

type ErrUnknownDomainOrCommand struct {
	Str string
}

func (u *ErrUnknownDomainOrCommand) Error() string {
	return fmt.Sprintf(`Unknown Command: '%s'.`, u.Str)
}

type ErrUnknownCommand struct {
	Cmd string
}

func (u *ErrUnknownCommand) Error() string {
	return fmt.Sprintf(`Unknown Command: '%s'.`, u.Cmd)
}

type ErrMissingArguments struct {
}

func (m *ErrMissingArguments) Error() string {
	return `Missing arguments`
}

type ErrNoCommandsGiven struct {
}

func (m *ErrNoCommandsGiven) Error() string {
	return `No commands given`
}

type ErrTODO struct {
	Kind string
	Name string
}

func (e *ErrTODO) Error() string {
	return fmt.Sprintf(`'%s' not implemented: '%s'.`, e.Kind, e.Name)
}
