package domains

import (
	"fmt"
	"strings"

	"tuxpa.in/t/wm/src/bsp"
	"tuxpa.in/t/wm/src/copies"
	"tuxpa.in/t/wm/src/sock"
)

type inject struct {
	xwm
}

type monitor_sel struct {
	monitorsel string
}

func (n *monitor_sel) tryMonitorSel(msg *sock.Msg) (bool, error) {
	// first split the str by .
	if !msg.HasNext() {
		return false, nil
	}
	str := msg.Peek()
	splt := strings.Split(str, ".")
	switch splt[0] {
	case "any", "first_ancestor",
		"last", "newest", "older", "newer",
		"focused", "pointed", "biggest", "smallest":
		n.monitorsel = str
		return true, nil
	}
	return false, nil
}

type desktop_sel struct {
	desktopsel string
}

func (n *desktop_sel) readDesktopSel(msg *sock.Msg) error {
	if !msg.HasNext() {
		return &copies.ErrMissingArguments{}
	}
	str := msg.Next()
	switch str {
	case "any", "first_ancestor",
		"last", "newest", "older", "newer",
		"focused", "pointed", "biggest", "smallest":
		n.desktopsel = str
	}
	return nil
}

type node_sel struct {
	nodesel string
}

func (n *node_sel) readNodeSel(msg *sock.Msg) error {
	if !msg.HasNext() {
		return &copies.ErrMissingArguments{}
	}
	str := msg.Next()
	switch str {
	case "any", "first_ancestor",
		"last", "newest", "older", "newer",
		"focused", "pointed", "biggest", "smallest":
		n.nodesel = str
	}
	return nil
}

type cmd struct {
	Command string
}

func (n *cmd) readCommand(msg *sock.Msg, c string) (bool, error) {
	if n.Command == "" {
		n.Command = c
		return msg.HasNext(), nil
	}
	return false, fmt.Errorf("multiple commands given")
}

type commandParser struct {
	routes map[string]string
}

func newCommandParser() *commandParser {
	return &commandParser{
		routes: map[string]string{},
	}
}

func (c *commandParser) addDef(name string, aliases ...string) *commandParser {
	c.routes["--"+strings.TrimSpace(name)] = name
	for _, a := range aliases {
		c.routes[strings.TrimSpace(a)] = name
	}
	return c
}

func (c *commandParser) parse(n *cmd, msg *sock.Msg) (bool, error) {
	arg := msg.Next()
	cmd, ok := c.routes[arg]
	if !ok {
		return false, fmt.Errorf(`unknown option: '%s'`, arg)
	}
	return n.readCommand(msg, cmd)
}

type xwm struct {
	XWM *bsp.XWM
}

func (x *xwm) SetXWM(z *bsp.XWM) {
	x.XWM = z
}
