package domains

import (
	"tuxpa.in/a/zlog/log"
	"tuxpa.in/t/wm/src/copies"
	"tuxpa.in/t/wm/src/sock"
)

type Config struct {
	UseNames bool

	configKey   string
	configValue string

	cmd
	inject
}

func (n *Config) Run(msg *sock.Msg) ([]byte, error) {
	if !msg.HasNext() {
		return nil, &copies.ErrMissingArguments{}
	}
	for {
		ok, err := n.parse(msg)
		if err != nil {
			return nil, err
		}
		if !ok {
			break
		}
	}
	if n.configValue == "" {
		str, err := n.XWM.W.Cfg.GetString(n.configKey)
		return []byte(str), err
	}
	err := n.XWM.W.Cfg.Set(n.configKey, n.configValue)
	if err != nil {
		return nil, err
	}
	log.Trace().Str("key", n.configKey).Any("string", n.configValue).Msg("config set")
	return nil, nil
}

func (n *Config) parse(msg *sock.Msg) (bool, error) {
	if !msg.HasNext() {
		return false, &copies.ErrMissingArguments{}
	}
	arg := msg.Next()
	switch arg {
	case "-d":
		return n.readCommand(msg, "desktops")
	case "-m":
		return n.readCommand(msg, "monitors")
	case "-n":
		return n.readCommand(msg, "nodes")
	default:
		n.configKey = arg
		if msg.HasNext() {
			n.configValue = msg.Next()
		}
		if msg.HasNext() {
			return false, &copies.ErrInvalidArgumentCount{Name: "2 or 3", Value: len(msg.Args())}
		}
		return false, nil
	}
}
