package domains

import (
	"fmt"

	"tuxpa.in/t/wm/src/copies"
	"tuxpa.in/t/wm/src/sock"
)

type Monitor struct {
	UseNames bool

	inject

	cmd

	monitor_sel
}

func (n *Monitor) Run(msg *sock.Msg) ([]byte, error) {
	if !msg.HasNext() {
		return nil, &copies.ErrMissingArguments{}
	}
	for {
		ok, err := n.parse(msg)
		if err != nil {
			return nil, err
		}
		if !ok {
			break
		}
	}
	switch n.Command {
	default:
		return nil, &copies.ErrTODO{
			Kind: "command",
			Name: n.Command,
		}
	}
}

func (n *Monitor) parse(msg *sock.Msg) (bool, error) {
	if !msg.HasNext() {
		return false, &copies.ErrMissingArguments{}
	}
	ok, err := n.tryMonitorSel(msg)
	if err != nil {
		return false, err
	}
	if ok {
		msg.Next()
	}
	arg := msg.Next()
	switch arg {
	case "--add-desktops", "-a":
		return n.readCommand(msg, "add-desktops")
	case "--focus", "-f":
		return n.readCommand(msg, "focus")
	case "--rectangle", "-g":
		return n.readCommand(msg, "rectangle")
	case "--remove", "-r":
		return n.readCommand(msg, "remove")
	case "--reorder-desktops", "-o":
		return n.readCommand(msg, "reorder-desktops")
	case "--reset-desktops", "-d":
		return n.readCommand(msg, "reset-desktops")
	case "--swap", "-s":
		return n.readCommand(msg, "swap")
	default:
		return false, fmt.Errorf(`unknown option: '%s'`, arg)
	}
}
