package domains

import (
	"bytes"
	"fmt"

	"tuxpa.in/t/wm/src/copies"
	"tuxpa.in/t/wm/src/sock"
)

type Query struct {
	UseNames bool

	inject

	cmd
}

func (n *Query) Run(msg *sock.Msg) ([]byte, error) {
	if !msg.HasNext() {
		return nil, &copies.ErrMissingArguments{}
	}
	for {
		ok, err := n.parse(msg)
		if err != nil {
			return nil, err
		}
		if !ok {
			break
		}
	}
	switch n.Command {
	case "desktops":
		return n.desktops(msg)
	case "monitors":
		return n.monitors(msg)
	default:
		return nil, &copies.ErrTODO{}
	}
}

func (n *Query) monitors(msg *sock.Msg) ([]byte, error) {
	o := new(bytes.Buffer)
	n.XWM.W.View(func() error {
		for _, v := range n.XWM.W.Monitors {
			if n.UseNames {
				o.WriteString(v.Name)
			} else {
				o.Write(v.HexId())
			}
			o.WriteRune('\n')
		}
		return nil
	})
	return o.Bytes(), nil
}
func (n *Query) desktops(msg *sock.Msg) ([]byte, error) {
	o := new(bytes.Buffer)
	o.WriteString("hi there")
	return o.Bytes(), nil
}

var queryParser = newCommandParser().
	addDef("desktop", "-d", "--desktops")

func (n *Query) parse(msg *sock.Msg) (bool, error) {
	if !msg.HasNext() {
		return false, &copies.ErrMissingArguments{}
	}
	arg := msg.Next()
	switch arg {
	case "--desktop", "-d", "--desktops", "-D":
		return n.readCommand(msg, "desktops")
	case "--monitor", "-m", "--monitors", "-M":
		return n.readCommand(msg, "monitors")
	case "--names":
		n.UseNames = true
	case "--node", "-n", "--nodes", "-N":
		return n.readCommand(msg, "nodes")
	case "--tree", "-T":
		return n.readCommand(msg, "tree")
	default:
		return false, fmt.Errorf(`unknown option: '%s'`, arg)
	}
	return msg.HasNext(), nil
}
