package handler

import (
	"fmt"

	"tuxpa.in/t/wm/src/bsp"
	"tuxpa.in/t/wm/src/copies"
	"tuxpa.in/t/wm/src/sock"
)

type Handler struct {
	XWM *bsp.XWM

	domains map[string]func() Domain
}

func AddDomain[T any, PT interface {
	Domain
	*T
}](h *Handler, name string) {
	if h.domains == nil {
		h.domains = map[string]func() Domain{}
	}
	h.domains[name] = func() Domain {
		domain := PT(new(T))
		domain.SetXWM(h.XWM)
		return domain
	}
}

func (h *Handler) AddDomainFunc(name string, fn func() Domain) {
	if h.domains == nil {
		h.domains = map[string]func() Domain{}
	}
	h.domains[name] = func() Domain {
		d := fn()
		d.SetXWM(h.XWM)
		return d
	}
}

func (h *Handler) Run(msg *sock.Msg) {
	resp, err := h.run(msg)
	if msg.Err(err) {
		return
	}
	err = msg.Reply(resp)
	if msg.Err(err) {
		return
	}
	msg.Reply(nil)
}

func (h *Handler) run(msg *sock.Msg) ([]byte, error) {
	if !msg.HasNext() {
		return nil, &copies.ErrMissingArguments{}
	}
	cmd := msg.Next()
	d, ok := h.domains[cmd]
	if !ok {
		return nil, &copies.ErrUnknownDomainOrCommand{Str: cmd}
	}
	return h.runDomain(cmd, msg, d())
}

func (h *Handler) runDomain(name string, msg *sock.Msg, d DomainRunner) ([]byte, error) {
	str, err := d.Run(msg)
	if err != nil {
		return nil, fmt.Errorf("%s: %w", name, err)
	}
	return str, nil
}
