package sock

import (
	"bufio"
	"net"
	"sync/atomic"
)

type Msg struct {
	c      *net.UnixConn
	args   []string
	closed atomic.Bool
	cur    int
}

func (m *Msg) HasNext() bool {
	return m.Has(0)
}
func (m *Msg) Peek() string {
	return m.Get(0)
}
func (m *Msg) Next() string {
	if m.Has(0) {
		m.cur = m.cur + 1
		return m.args[m.cur-1]
	}
	return ""
}
func (m *Msg) Has(i int) bool {
	return len(m.args) > (i + m.cur)
}
func (m *Msg) Get(i int) string {
	if m.Has(i) {
		return m.args[i+m.cur]
	}
	return ""
}
func (m *Msg) Args() []string {
	if m.HasNext() {
		return m.args[m.cur:]
	}
	return nil
}
func (m *Msg) Err(e error) bool {
	if e == nil {
		return false
	}
	if !m.closed.CompareAndSwap(false, true) {
		return true
	}
	defer m.c.Close()
	wr := bufio.NewWriter(m.c)
	wr.Write([]byte{7})
	wr.Write([]byte(e.Error()))
	wr.Write([]byte("\n"))
	wr.Flush()
	return true
}
func (m *Msg) Reply(xs []byte) error {
	if !m.closed.CompareAndSwap(false, true) {
		return nil
	}
	defer m.c.Close()
	wr := bufio.NewWriter(m.c)
	_, err := wr.Write(xs)
	if err != nil {
		return err
	}
	if len(xs) != 0 && xs[len(xs)-1] != '\n' {
		wr.Write([]byte("\n"))
	}
	wr.Write([]byte{0})
	wr.Flush()
	return nil
}
