package sock

import (
	"bytes"
	"fmt"
	"net"
	"strings"
)

type SockListener struct {
	l *net.UnixListener

	ch chan *Msg
}

func (s *SockListener) Msg() <-chan *Msg {
	return s.ch
}

func Server(addr *net.UnixAddr) (*SockListener, error) {
	s := &SockListener{}
	s.ch = make(chan *Msg, 1)
	conn, err := net.ListenUnix("unix", addr)
	if err != nil {
		return nil, err
	}
	s.l = conn
	go func() {
		for {
			conn, err := s.l.AcceptUnix()
			if err != nil {
				continue
			}
			go func() {
				err = s.acceptConn(conn)
				if err != nil {
					fmt.Printf("fail accept conn: %s\n", err)
				}
			}()
		}
	}()
	return s, nil
}
func (s *SockListener) Close() error {
	err := s.l.Close()
	return err
}
func (s *SockListener) acceptConn(c *net.UnixConn) error {
	msg := &Msg{}
	bts := make([]byte, 4096)
	n, err := c.Read(bts)
	if err != nil {
		return err
	}
	bts = bts[:n]
	splt := bytes.Split(bts, []byte{0})
	for _, v := range splt {
		if len(strings.TrimSpace(string(v))) > 0 {
			msg.args = append(msg.args, string(v))
		}
	}
	msg.c = c
	s.ch <- msg
	return nil
}
