package sock

import (
	"bufio"
	"fmt"
	"io"
	"net"
	"os"

	"github.com/jezek/xgb"
)

const SOCKET_PATH_TPL = "/tmp/bspwm%s_%d_%d-socket"

const SOCKET_ENV_VAR = "BSPWM_SOCKET"

type Sock struct {
	C *net.UnixConn
}

func (s *Sock) Send(args ...string) (string, error) {
	if len(args) == 0 {
		return "", s.C.Close()
	}
	wr := bufio.NewWriter(s.C)
	for _, msg := range args {
		wr.WriteString(msg)
		wr.WriteByte(0)
	}
	wr.Flush()
	s.C.CloseWrite()
	bts, err := io.ReadAll(s.C)
	if err != nil {
		return "", err
	}
	s.C.CloseRead()
	if len(bts) > 0 {
		if bts[0] == 7 {
			return "", fmt.Errorf(string(bts[1:]))
		}
	}
	return string(bts), nil
}

func Client(path string) (*Sock, error) {
	xc, err := xgb.NewConn()
	if err != nil {
		return nil, err
	}
	defer xc.Close()
	if path == "" {
		path = os.Getenv(SOCKET_ENV_VAR)
	}
	if path == "" {
		path = fmt.Sprintf(SOCKET_PATH_TPL, "", xc.DisplayNumber, xc.DefaultScreen)
	}
	addr, err := net.ResolveUnixAddr("unix", path)
	if err != nil {
		return nil, err
	}
	conn, err := net.DialUnix("unix", nil, addr)
	if err != nil {
		return nil, err
	}
	return &Sock{C: conn}, nil
}
